
public class JavaParser extends JavaOriginal {
    private final boolean supportAssertion;
    public JavaParser(boolean supportAssertion) {
        this.supportAssertion = supportAssertion;
    }
    
    protected LexicalAnalyzer createLexicalAnalyzer(String sourceName, CharSequence text, int tabStop) throws ParseException {
        StringBuffer buffer = new StringBuffer(text.length());
    loop:
        for (int i = 0; i < text.length();) {
            char ch = text.charAt(i++);
            if (ch != '\\' || i >= text.length()) {
                buffer.append(ch);
            } else {
                char ch2 = text.charAt(i++);
                if (ch2 != 'u') {
                    buffer.append(ch);
                    buffer.append(ch2);
                } else {
                    int start = i - 1;
                    // skip u's.
                    for (; i < text.length(); i++) {
                        ch2 = text.charAt(i);
                        if (ch2 != 'u')
                            break;
                    }
                    int end = i + 4;
                    if (i + 4 > text.length()) {
                        end = text.length();
                    } else {
                        try {
                            int code = Integer.parseInt(text.subSequence(i, end).toString(), 16);
                            buffer.append((char) code);
                            i = end;
                            continue loop;
                        } catch (NumberFormatException x) {
                        }
                    }
                    throw new ParseException("illigal Unicode escape: " + text.subSequence(start, end), null, start, -1, -1);
                }
            }
        }
        final LexicalAnalyzer defaultAnalyzer = super.createLexicalAnalyzer(sourceName, buffer, tabStop);
        if (supportAssertion) {
            return defaultAnalyzer;
        } else {
            return new LexicalAnalyzer() {
                public Token next() throws ParseException {
                    Token result = defaultAnalyzer.next();
                    if (result.getSymbolID() == TOKEN_ASSERT) {
                        result = new Default.Token(TOKEN_IDENTIFIER, result.getImage(), result);
                    }
                    return result;
                }
            };
        }
    }
}

